# vim: set fileencoding=utf-8 :
from django.contrib import admin
from taxi.models import *

class CarAdmin(admin.ModelAdmin):
	list_display = ('reg_number', 'mark', 'model', 'color_type', 'color', 'build_year')
	search_fields = ('reg_number', 'mark', 'model')
	list_filter = ('color_type', 'color', 'mark')

class CabbyAdmin(admin.ModelAdmin):
	fieldsets = (
		(u'Основная информация', {'fields': (
						'login',
						'password',
						'lastname',
						'firstname',
						'middlename',
						'photo',
						'work_tel',
						'mobile_tel',
						'email',
						'jabber')}),
		(u'Счет', 		{'fields': (
						'company',
						'balance',
						)}),
		(u'Доп. информация', {'fields': (
						'car',
						'status',
						'adr_city',
						'adr_raion',
						'adr_street',
						'adr_build',
						)}),
	)

	list_display = ('lastname', 'firstname', 'middlename', 'company', 'balance', 'status', 'get_full_addr')
	ordering = ('lastname',)
	search_fields = ('lastname',)
	list_filter = ('status',)
	def save_model(self, request, obj, form, change):
		obj.save(admin=request.user)
		form.save()

class TransferAdmin(admin.ModelAdmin):
	fieldsets = (
		(u'Основная информация', {'fields': (
						'date_time',
						'client',
						'cost',
						'cabby',
						'cabby_payment',
						'type',
						'status',
						'confirmed')}),
		(u'Адреса', 		{'fields': (
						'adr_city_in',
						'adr_raion_in',
						'adr_street_in',
						'adr_build_in',
						'adr_porch_in',
						'adr_city_out',
						'adr_raion_out',
						'adr_street_out',
						'adr_build_out',
						'adr_porch_out',
						)}),
	)

	list_display = ('date_time', 'cabby', 'type', 'status', 'confirmed')
	search_fields = ('cabby',)
	ordering = ('date_time',)
	list_filter = ('type', 'status', 'confirmed')
	date_hierarchy = 'date_time'
	def save_model(self, request, obj, form, change):
		obj.save(admin=request.user)
		form.save()


class ServiceTaxiUparamAdmin(admin.ModelAdmin):
	pass

class ServiceTaxiFilterAdmin(admin.ModelAdmin):
	fieldsets = (
		(u'Основная информация', {'fields': (
						'tariff',
						'pay_km',
						'pay_km_outside',
						'pay_min',
						'km_min',
						'pay_time_min',
						'pay_hour',
						'pay_refusal',
						'pay_wait',
						'bonus_trans',
						'bonus_discount_percent',
						'director_percent',
						)}),
	)

	list_display = ('tariff', 'director_percent', 'pay_km', 'km_min', 'pay_min', 'pay_wait')
	search_fields = ('tariff',)
	list_filter = ('director_percent', 'pay_min')

class ContractAdmin(admin.ModelAdmin):
	date_hierarchy = 'start_date_time'

class DirectorAdmin(admin.ModelAdmin):
	pass

class CompanyAdmin(admin.ModelAdmin):
	pass

class CompanyTelAdmin(admin.ModelAdmin):
	pass

### History admin models
class TransferHistoryAdmin(admin.ModelAdmin):
	list_display = ('date_time', 'transfer', 'status', 'confirmed', 'admin')
	ordering = ('date_time',)
	list_filter = ('date_time', 'admin')

class CabbyHistoryAdmin(admin.ModelAdmin):
	pass
### END History admin models

admin.site.register(Car, CarAdmin)
admin.site.register(Cabby, CabbyAdmin)
admin.site.register(ServiceTaxiStat, TransferAdmin)

admin.site.register(TransferHistory, TransferHistoryAdmin)
admin.site.register(CabbyHistory, CabbyHistoryAdmin)

admin.site.register(ServiceTaxiFilter, ServiceTaxiFilterAdmin)
admin.site.register(ServiceTaxiUparam, ServiceTaxiUparamAdmin)

admin.site.register(Contract, ContractAdmin)
admin.site.register(Director, DirectorAdmin)
admin.site.register(Company, CompanyAdmin)
admin.site.register(CompanyTel, CompanyTelAdmin)
