# vim: set fileencoding=utf-8 :
from django.db import models
from katrin.models import *
from katrin import service
from django.contrib.auth.models import User
from datetime import datetime

import decimal


CABBY_STATUS = (
	(0, u'Не доступен'),
	(1, u'Свободен'),
	(2, u'В дороге к клиенту'),
	(3, u'Ожидание клиента'),
	(4, u'В пути с клиентом'),
)

TASK_STATUS = (
	(0, u'Не обработан'),
	(1, u'Назначение водителя'),
	(2, u'Выполнение'),
	(3, u'Завершено'),
)

TASK_TYPE = (
	(0, u'Легковая а/м'),
	(1, u'Грузовая а/м'),
	(2, u'Микроавтобус'),
	(3, u'Автобус'),
	(4, u'Привоз товаров'),
)


def get_full_addr(city=None, raion=None, street=None, build=None, porch=None):
	adr = u''
	if city:
		adr = "%s." % city
	if street and build:
		adr += " %s, %s" % (street, build)
		if porch:
			adr += "[%s]" % porch
	if raion:
		adr += " (%s)" % raion
	if adr and adr[0] == ' ':
		adr = adr[1:]
	return adr



class Contract(models.Model):
	number = models.IntegerField(u'Номер')
	start_date_time = models.DateTimeField(u'Начало контракта')
	end_date_time = models.DateTimeField(u'Окончание контракта')
	max_cabbies = models.IntegerField(u'Максимальное число водителей', blank=True, null=True)
	our_percent = models.DecimalField(u'Наш процент', max_digits=5, decimal_places=2, default=0.0)
	month_payment = models.DecimalField(u'Месячный платеж', max_digits=10, decimal_places=2, default=0.0)
	class Meta:
		verbose_name = u'контракт'
		verbose_name_plural = u'Контракты с директорами'
	def __unicode__(self):
		return "%s (%s - %s)" % (self.number, self.start_date_time, self.end_date_time)

class Director(Human):
	class Meta:
		verbose_name = u'директор'
		verbose_name_plural = u'Директора'
	def save(self, admin=None):
		super(Director, self).save()
		self.user.groups.add(Group.objects.get(name='taxi_directors'))
		self.user.save()

class Company(models.Model):
	title = models.CharField(u'Название', max_length=80)
	tariff = models.ForeignKey(Tariff, verbose_name=u'Тариф', blank=True, null=True, help_text=u'Оставьте не заполненным чтобы система сама создала тариф')
	director = models.ForeignKey(Director, verbose_name=u'Директор')
	contract = models.ForeignKey(Contract, verbose_name=u'Контракт')
	class Meta:
		verbose_name = u'компания'
		verbose_name_plural = u'Компании'
	def __unicode__(self):
		return "%s (%s)" % (self.title, self.director)
	def save(self):
		# Create tariff for this company if it not pointed
		if not self.tariff:
			tariff = Tariff(title=self.title,
					service=Service.objects.filter(codename='taxi')[0],
					day_payment=0,
					month_payment=0)
			tariff.save()
			self.tariff = tariff
		super(Company, self).save()
		# Create filter for this company tariff if it doesn't exist
		if not ServiceTaxiFilter.objects.filter(tariff=self.tariff).count():
			filter = ServiceTaxiFilter(title=self.title,
						tariff=self.tariff,
						priority=0,
						store_stat=1,
						pay_km=0,
						pay_km_outside=0,
						pay_min=0,
						km_min=0,
						pay_time_min=0,
						pay_hour=0,
						pay_refusal=0,
						pay_wait=0,
						bonus_trans=0,
						bonus_discount_percent=0,
						director_percent=0)
			filter.save()

class CompanyTel(models.Model):
	number = models.CharField(u'Номер телефона', max_length=20)
	company = models.ForeignKey(Company, verbose_name=u'Компания', related_name='tel_set')
	class Meta:
		verbose_name = u'телефон компании'
		verbose_name_plural = u'Телефоны компаний'
	def __unicode__(self):
		return "%s - %s" % (self.number, self.company)

CAR_COLOR_TYPES = (
	(0, u'Темно'),
	(1, u'Светло'),
	(2, u'Металлик'),
)

CAR_COLORS = (
	(0, u'Белый'),
	(1, u'Черный'),
	(2, u'Красный'),
	(3, u'Оранжевый'),
	(4, u'Желтый'),
	(5, u'Зеленый'),
	(6, u'Голубой'),
	(7, u'Синий'),
	(8, u'Фиолетовый'),
)

class Car(models.Model):
	company = models.ForeignKey(Company, verbose_name=u'Компания')
	reg_number = models.CharField(u'Регистрационный номер', max_length=20)
	mark = models.CharField(u'Марка', max_length=20)
	model = models.CharField(u'Модель', max_length=20)
	color_type = models.IntegerField(u'Тип цвета', choices=CAR_COLOR_TYPES)
	color = models.IntegerField(u'Цвет', choices=CAR_COLORS)
	build_year = models.IntegerField(u'Год выпуска', null=True, blank=True)
	class Meta:
		verbose_name = u'автомобиль'
		verbose_name_plural = u'Автомобили'
	def __unicode__(self):
		return "%s %s (%s)" % (self.mark, self.model, self.reg_number)

class Cabby(Human):
	company = models.ForeignKey(Company, verbose_name=u'Компания')
	car = models.ForeignKey(Car, verbose_name=u'Автомобиль')
	status = models.IntegerField(u'Статус', choices=CABBY_STATUS, default=0)
	adr_city = models.CharField(u'Город', max_length=200, null=True, blank=True)
	adr_raion = models.CharField(u'Район', max_length=200, null=True, blank=True)
	adr_street = models.CharField(u'Улица', max_length=200, null=True, blank=True)
	adr_build = models.CharField(u'Здание', max_length=200, null=True, blank=True)
	class Meta:
		verbose_name = u'водитель'
		verbose_name_plural = u'Водители'
	def save(self, admin=None):
		h = CabbyHistory(
				date_time = datetime.now(),
				admin = admin,

				balance = self.balance,
				status = self.status,
			)
		super(Cabby, self).save()
		h.cabby = self
		h.save()

		self.user.groups.add(Group.objects.get(name='taxi_cabbies'))
		self.user.save()
	def get_full_addr(self):
		return get_full_addr(self.adr_city, self.adr_raion, self.adr_street, self.build)
	get_full_addr.short_description = u'Полный адрес'

class ServiceTaxiFilter(service.ServiceFilter):
	pay_km = models.DecimalField(u'Плата за километр', max_digits=10, decimal_places=2)
	pay_km_outside = models.DecimalField(u'Плата за километр за чертой города', max_digits=10, decimal_places=2)
	pay_min = models.DecimalField(u'Минимальная оплата', max_digits=10, decimal_places=2)
	km_min = models.DecimalField(u'Минимальный километраж', max_digits=10, decimal_places=2)
	pay_time_min = models.DecimalField(u'Оплата вызова на время', max_digits=10, decimal_places=2)
	pay_hour = models.DecimalField(u'Почасовая оплата', max_digits=10, decimal_places=2)
	pay_refusal = models.DecimalField(u'Стоимость отказа', max_digits=10, decimal_places=2, help_text=u'Отказ от поездки при своевременной подаче такси')
	pay_wait = models.DecimalField(u'Ожидание клиента', max_digits=10, decimal_places=2, help_text=u'руб./мин.')
	bonus_trans = models.IntegerField(u'Бонусная поездка', help_text=u'Каждая N-ая поездка для клиента будет со скидкой')
	bonus_discount_percent = models.DecimalField(u'Скидка бонусной поездки в процентах', max_digits=10, decimal_places=2)
	director_percent = models.DecimalField(u'Процент с поездки хозяину такси', max_digits=10, decimal_places=2)
	class Meta:
		db_table = u'service_taxi_filters'
		verbose_name = u'параметр тарифа такси'
		verbose_name_plural = u'Параметры тарифов такси'

class ServiceTaxiStat(service.ServiceStat):
	"""Перевозка"""
	cabby = models.ForeignKey(Cabby, verbose_name=u'Водитель', blank=True, null=True, help_text=u'Если это поле не заполнено, то система сама назначит водителя', related_name='transfer_set')
	adr_city_in = models.CharField(u'Город посадки', max_length=80)
	adr_raion_in = models.CharField(u'Район посадки', max_length=80)
	adr_street_in = models.CharField(u'Улица посадки', max_length=80)
	adr_build_in = models.CharField(u'Дом посадки', max_length=80)
	adr_porch_in = models.CharField(u'Подъезд посадки', max_length=80, blank=True, null=True)
	adr_city_out = models.CharField(u'Город высадки', max_length=80)
	adr_raion_out = models.CharField(u'Район посадки', max_length=80)
	adr_street_out = models.CharField(u'Улица высадки', max_length=80)
	adr_build_out = models.CharField(u'Дом высадки', max_length=80)
	adr_porch_out = models.CharField(u'Подъезд высадки', max_length=80, blank=True, null=True)
	cabby_payment = models.DecimalField(u'Комиссия с водителя', max_digits=10, decimal_places=2, default=0.0)
	type = models.IntegerField(u'Тип', choices=TASK_TYPE, default=0)
	status = models.IntegerField(choices=TASK_STATUS, verbose_name=u'Статус', default=0)
	confirmed = models.NullBooleanField(u'Принято водителем')
	def get_full_addr_in(self):
		return get_full_addr(self.adr_city_in, self.adr_raion_in, self.adr_street_in, self.adr_build_in, self.adr_porch_in)
	def get_full_addr_out(self):
		return get_full_addr(self.adr_city_out, self.adr_raion_out, self.adr_street_out, self.adr_build_out, self.adr_porch_out)
	def __unicode__(self):
		return u"Перевозка %s" % self.id
	class Meta:
		db_table = u'service_taxi_stats'
		verbose_name = u'перевозка'
		verbose_name_plural = u'Перевозки'
	def save(self, admin=None):
		if not self.filter:
			self.filter = ServiceTaxiFilter.objects.filter(tariff=self.cabby.company.tariff)[0].pk
		th = TransferHistory(
				date_time = datetime.now(),
				admin = admin,

				status = self.status,
				confirmed = self.confirmed,
			)
		# Payment to cabby when transfer is success
		if self.status == 3 and self.id and self.cabby:
			old_transfer = ServiceTaxiStat.objects.get(pk=self.id)
			# Only if it is first set transfer status to success
			if TransferHistory.objects.filter(transfer=old_transfer, status=3).count() == 0:
				cost = self.cost
				cabby = self.cabby

				e_base_payment = cabby.remote_base_payment

				cabby.balance += (cost * e_percent / decimal.Decimal(100) + e_base_payment)
				cabby.save()

				if dispatcher:
					dispatcher.balance += (cost * d_percent / decimal.Decimal(100) + d_base_payment)
					dispatcher.save()
				client = self.contract.client
				client.balance -= cost
				client.save()

		super(ServiceTaxiStat, self).save()
		th.transfer = self
		th.save()
		if not self.cabby:
			self.assign_cabbies_to_new_transfers()

	def assign_cabbies_to_new_transfers(self):
		"""Assign cabbies for transfers without cabby"""
		for transfer in Transfer.objects.filter(cabby=None):
			cabbies = transfer.contract.client.cabby_set.filter(status__gte=1).order_by('status')
			if cabbies:
				transfer.cabby = cabbies[0]
				transfer.status = 0
				transfer.save()

	def assign_cabbies_to_rejected_transfers(self):
		"""Reassign cabbies for rejected transfers"""
		for transfer in Transfer.objects.filter(confirmed=False):
			cabbies =  transfer.contract.client.cabby_set.all()
			if cabbies:
				continue
			try:
				old_cabby_index = cabbies.index(transfer.cabby)
				if len(cabbies) <= old_cabby_index + 1:
					new_cabby_index = 0
				else:
					new_cabby_index = old_cabby_index + 1
			except:
				new_cabby_index = 0
			transfer.cabby = cabbies[new_cabby_index]
			transfer.confirmed = None
			transfer.status = 0
			transfer.save()

class ServiceTaxiUparam(service.ServiceUparam):
	class Meta:
		db_table = u'service_taxi_uparams'
		verbose_name = u'профиль клиента'
		verbose_name_plural = u'Профили клиентов'
	def __unicode__(self):
		return u"%s" % self.client
	def save(self, admin=None):
		super(ServiceTaxiUparam, self).save()
		self.client.user.groups.add(Group.objects.get(name='taxi_clients'))
		self.client.user.save()


### History models
from datetime import datetime
class TransferHistory(models.Model):
	id = models.AutoField(primary_key=True)
	date_time = models.DateTimeField(u'Дата и время', default=datetime.now())
	admin = models.ForeignKey(User, verbose_name=u'Админ', null=True, blank=True)

	transfer = models.ForeignKey(ServiceTaxiStat, verbose_name=u'Вызов')
	status = models.IntegerField(choices=TASK_STATUS, verbose_name=u'Статус')
	confirmed = models.NullBooleanField(u'Принята водителем')

	class Meta:
		verbose_name = u'история заданий'
		verbose_name_plural = u'История заданий'

class CabbyHistory(models.Model):
	id = models.AutoField(primary_key=True)
	date_time = models.DateTimeField(default=datetime.now())
	admin = models.ForeignKey(User, verbose_name=u'Админ', null=True, blank=True)

	cabby = models.ForeignKey(Cabby, verbose_name=u'Водитель')

	balance = models.DecimalField(u'Баланс', max_digits=10, decimal_places=2, default=0.0)
	status = models.IntegerField(u'Статус', choices=CABBY_STATUS, default=0)

	class Meta:
		verbose_name = u'история водителей'
		verbose_name_plural = u'История водителей'

### END History models


